<?php
namespace swg ;

class SecretWordGame
{

    private const HIDDEN_CHAR = '?' ;

    private array $secret ;

    public function __construct(string $secret)
    {
        $this->secret = str_split(strtolower($secret), 1);
    }

    public function try(?array $word=null): array{

        $result = "" ;
        $index = 0 ;

        if (isset($word)){
            for ($i=0 ; $i<count($word) ; $i++) $word[$i] = strtolower($word[$i]) ;
            for ($index; $index < count($this->secret); $index++) {
                if($this->secret[$index] == " "){
                    $result .= " " ;
                } elseif (isset($word[$index])) {
                    if ($this->secret[$index] == $word[$index]) {
                        $result .= $this->secret[$index];
                    } else {
                        $result .= self::HIDDEN_CHAR;
                    }
                } else {
                    break;
                }
            }
        }

        if ($index < count($this->secret)) {
            for ($index; $index < count($this->secret); $index++) {
                if($this->secret[$index] == " ") $result .= " " ;
                else $result .= self::HIDDEN_CHAR;
            }
        }

        $win = join("", $word) == join("", $this->secret) ;
        return $this->generateResponse($word, $win, $result) ;

    }

    private function generateResponse(array $word, bool $win, string $result){
        return array(
            'word' => $word,
            'win' => $win,
            'result' => $result
        );
    }

    public function generateInput(?array $response): void{
        $lastWord = htmlspecialchars(join("", $response['word'])) ;
        echo "<div id='secret-word'>".$response['result']."</div>" ;
        echo <<<EOT
            <input type='text' name='word' id='word'
                    value='$lastWord' ><br>
            EOT;
    }

    private function generateInputLetter(int $i, string $letter, $old="") : string{
        $str = "" ;
        switch ($letter){
            case self::HIDDEN_CHAR:
                $str .= "<input type='text' class='secret-letter-input' name='letter[$i]' value='".htmlspecialchars($old)."'>\n" ;
                break ;
            case " ":
                $str .= "<input type='hidden' name='letter[$i]' value=' '>\n" ;
                $str .= "<div class='secret-letter-separator'></div>\n" ;
                break ;
            default:
                $str .= "<input type='text' class='secret-letter-input' value='$letter' name='letter[$i]' disabled>\n" ;
                $str .= "<input type='hidden' name='letter[$i]' value='$letter'>\n" ;
                break ;
        }
        return $str ;
    }

    public function generateInputV2(?array $response): void{
        $lastWord = $response['word'] ;
        if (isset($lastWord) and !empty($lastWord)){
            $lastResult = $response['result'] ;
            for ($i=0 ; $i<strlen($lastResult) ; $i++){
                echo $this->generateInputLetter($i, $lastResult[$i], isset($lastWord[$i]) ? $lastWord[$i] : null) ;
            }
        }else{
            for ($i=0 ; $i<count($this->secret) ; $i++){
                echo $this->generateInputLetter($i, ($this->secret[$i] === " " ? " " : self::HIDDEN_CHAR)) ;
            }
        }
    }

    public function generateWin() : void{
        echo "<div id='secret-word'>".join("", $this->secret)."</div>" ;
        echo "<div id='secret-word-win'>!!! YOU WIN !!!</div>" ;
    }

}